@@ -93,21 +93,6 @@ def pull_image(name):
         LOG.debug(stderr)
 
 
-def get_config_base(prefix, volume):
-    # crawl the volume's path upwards until we find the
-    # volume's base, where the hashed config file resides
-    path = volume
-    base = prefix.rstrip(os.path.sep)
-    base_generated = os.path.join(base, 'puppet-generated')
-    while path.startswith(prefix):
-        dirname = os.path.dirname(path)
-        if dirname == base or dirname == base_generated:
-            return path
-        else:
-            path = dirname
-    raise ValueError("Could not find config's base for '%s'" % volume)
-
-
 def match_config_volumes(prefix, config):
     # Match the mounted config volumes - we can't just use the
     # key as e.g "novacomute" consumes config-data/nova
@@ -121,8 +106,8 @@ def match_config_volumes(prefix, config):
             )
         )
         raise
-    return sorted([get_config_base(prefix, v.split(":")[0])
-                   for v in volumes if v.startswith(prefix)])
+    return sorted([os.path.dirname(v.split(":")[0]) for v in volumes if
+                   v.startswith(prefix)])
 
 
 def get_config_hash(config_volume):
@@ -215,12 +200,7 @@ def mp_puppet_config(*args):
             CLI_CMD,
             'run',
             '--user',
-            # Using '0' and not 'root' because it seems podman is susceptible to a race condition
-            # https://bugzilla.redhat.com/show_bug.cgi?id=1776766 and
-            # https://bugs.launchpad.net/tripleo/+bug/1803544 which are still lurking
-            # by using a UID we skip the code that parses /etc/passwd entirely and basically
-            # paper over this issue
-            '0',
+            'root',
             '--name',
             uname,
             '--env',
@@ -265,6 +245,7 @@ def mp_puppet_config(*args):
             # Syslog socket for puppet logs
             '--volume', '/dev/log:/dev/log:rw'
         ]
+
         # Remove container by default after the run
         # This should mitigate the "ghost container" issue described here
         # https://bugzilla.redhat.com/show_bug.cgi?id=1747885
@@ -393,8 +374,9 @@ def infile_processing(infiles):
         config_hash = '-'.join(config_hashes)
         if config_hash:
             LOG.debug(
-                "Updating config hash for %s, hash=%s" % (
+                "Updating config hash for %s, config_volume=%s hash=%s" % (
                     c_name,
+                    config_volume,
                     config_hash
                 )
             )
@@ -597,4 +579,4 @@ if __name__ == '__main__':
     infile_processing(infiles=glob.glob(STARTUP_CONFIGS))
 
     if not SUCCESS:
-        raise SystemExit(1)
+        raise SystemExit()
