@@ -49,6 +49,8 @@ REQUIRED_DOCKER_SECTIONS = ['service_name', 'docker_config', 'puppet_config',
 OPTIONAL_DOCKER_SECTIONS = ['container_puppet_tasks', 'upgrade_tasks',
                             'deploy_steps_tasks',
                             'pre_upgrade_rolling_tasks',
+                            'fast_forward_upgrade_tasks',
+                            'fast_forward_post_upgrade_tasks',
                             'post_upgrade_tasks', 'update_tasks',
                             'post_update_tasks', 'service_config_settings',
                             'host_prep_tasks', 'metadata_settings',
@@ -60,6 +62,8 @@ OPTIONAL_DOCKER_SECTIONS = ['container_puppet_tasks', 'upgrade_tasks',
                             'external_update_tasks', 'external_upgrade_tasks']
 # ansible tasks cannot be an empty dict or ansible is unhappy
 ANSIBLE_TASKS_SECTIONS = ['upgrade_tasks', 'pre_upgrade_rolling_tasks',
+                          'fast_forward_upgrade_tasks',
+                          'fast_forward_post_upgrade_tasks',
                           'post_upgrade_tasks', 'update_tasks',
                           'post_update_tasks', 'host_prep_tasks',
                           'external_deploy_tasks',
@@ -101,7 +105,6 @@ PARAMETER_DEFINITION_EXCLUSIONS = {
     'DesignateProducerLoggingSource': ['default'],
     'DesignateSinkLoggingSource': ['default'],
     'DesignateWorkerLoggingSource': ['default'],
-    'Ec2ApiLoggingSource': ['default'],
     'GlanceApiLoggingSource': ['default'],
     'GnocchiApiLoggingSource': ['default'],
     'HeatApiCfnLoggingSource': ['default'],
@@ -187,7 +190,6 @@ PARAMETER_DEFINITION_EXCLUSIONS = {
     }
 
 PREFERRED_CAMEL_CASE = {
-    'ec2api': 'Ec2Api',
     'haproxy': 'HAProxy',
     'metrics-qdr': 'MetricsQdr'
 }
@@ -219,10 +221,6 @@ VALIDATE_DOCKER_OVERRIDE = {
   # deploy container
   './deployment/rabbitmq/rabbitmq-messaging-notify-shared-puppet.yaml': False,
 }
-VALIDATE_DOCKER_PUPPET_CONFIG_OVERRIDE = {
-    # inherits from nova-conductor
-  './deployment/nova/nova-manager-container-puppet.yaml': False,
-}
 DEPLOYMENT_RESOURCE_TYPES = [
     'OS::Heat::SoftwareDeploymentGroup',
     'OS::Heat::StructuredDeploymentGroup',
@@ -246,6 +244,9 @@ WORKFLOW_TASKS_EXCLUSIONS = [
     './deployment/ceph-ansible/ceph-base.yaml',
     './deployment/ceph-ansible/ceph-mon.yaml',
     './deployment/ceph-ansible/ceph-mgr.yaml',
+    './docker/services/skydive/skydive-base.yaml',
+    './docker/services/skydive/skydive-agent.yaml',
+    './docker/services/skydive/skydive-analyzer.yaml',
 ]
 
 
@@ -270,7 +271,7 @@ def to_camel_case(string):
 def get_base_endpoint_map(filename):
     try:
         with open(filename, 'r') as f:
-            tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+            tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
         return tpl['parameters']['EndpointMap']['default']
     except Exception:
         print(traceback.format_exc())
@@ -280,7 +281,7 @@ def get_base_endpoint_map(filename):
 def get_endpoint_map_from_env(filename):
     try:
         with open(filename, 'r') as f:
-            tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+            tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
         return {
             'file': filename,
             'map': tpl['parameter_defaults']['EndpointMap']
@@ -296,7 +297,7 @@ def validate_endpoint_map(base_map, env_map):
 
 def validate_role_name(filename):
     with open(filename, 'r') as f:
-        tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     role_data = tpl[0]
     if role_data['name'] != os.path.basename(filename).split('.')[0]:
@@ -312,7 +313,7 @@ def validate_hci_compute_services_default(env_filename, env_tpl):
     roles_filename = os.path.join(os.path.dirname(env_filename),
                                   '../roles/Compute.yaml')
     with open(roles_filename, 'r') as f:
-        roles_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        roles_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     for role in roles_tpl:
         if role['name'] == 'Compute':
@@ -328,7 +329,7 @@ def validate_hci_computehci_role(hci_role_filename, hci_role_tpl):
     compute_role_filename = os.path.join(os.path.dirname(hci_role_filename),
                                          './Compute.yaml')
     with open(compute_role_filename, 'r') as f:
-        compute_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        compute_role_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     compute_role_services = compute_role_tpl[0]['ServicesDefault']
     for role in hci_role_tpl:
@@ -346,7 +347,7 @@ def validate_controller_dashboard(filename, tpl):
     control_role_filename = os.path.join(os.path.dirname(filename),
                                          './Controller.yaml')
     with open(control_role_filename, 'r') as f:
-        control_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        control_role_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     control_role_services = control_role_tpl[0]['ServicesDefault']
     for role in tpl:
@@ -359,30 +360,13 @@ def validate_controller_dashboard(filename, tpl):
     return 0
 
 
-def validate_controller_storage_nfs(filename, tpl, exclude_service=()):
-    control_role_filename = os.path.join(os.path.dirname(filename),
-                                         './Controller.yaml')
-    with open(control_role_filename, 'r') as f:
-        control_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
-
-    control_role_services = control_role_tpl[0]['ServicesDefault']
-    for role in tpl:
-        if role['name'] == 'ControllerStorageNfs':
-            services = [x for x in role['ServicesDefault'] if (x not in exclude_service)]
-            if sorted(services) != sorted(control_role_services):
-                print('ERROR: ServicesDefault in %s is different from '
-                      'ServicesDefault in roles/Controller.yaml' % filename)
-                return 1
-    return 0
-
-
 def validate_hci_role(hci_role_filename, hci_role_tpl):
     role_files = ['HciCephAll', 'HciCephFile', 'HciCephMon', 'HciCephObject']
     if hci_role_filename in ['./roles/' + x + '.yaml' for x in role_files]:
         compute_role_filename = \
             os.path.join(os.path.dirname(hci_role_filename), './Compute.yaml')
         with open(compute_role_filename, 'r') as f:
-            compute_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+            compute_role_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
         compute_role_services = compute_role_tpl[0]['ServicesDefault']
         for role in hci_role_tpl:
@@ -421,7 +405,7 @@ def validate_ceph_role(ceph_role_filename, ceph_role_tpl):
         ceph_storage_role_filename = \
             os.path.join(os.path.dirname(ceph_role_filename), './CephStorage.yaml')
         with open(ceph_storage_role_filename, 'r') as f:
-            ceph_storage_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+            ceph_storage_role_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
         ceph_storage_role_services = ceph_storage_role_tpl[0]['ServicesDefault']
         for role in ceph_role_tpl:
@@ -452,7 +436,7 @@ def validate_controller_no_ceph_role(filename, tpl):
     control_role_filename = os.path.join(os.path.dirname(filename),
                                          './Controller.yaml')
     with open(control_role_filename, 'r') as f:
-        control_role_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        control_role_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     control_role_services = control_role_tpl[0]['ServicesDefault']
     for role in tpl:
@@ -476,7 +460,7 @@ def validate_with_compute_role_services(role_filename, role_tpl, exclude_service
     cmpt_filename = os.path.join(os.path.dirname(role_filename),
                                  './Compute.yaml')
     with open(cmpt_filename, 'r') as f:
-        cmpt_tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+        cmpt_tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
     cmpt_services = cmpt_tpl[0]['ServicesDefault']
     cmpt_services = [x for x in cmpt_services if (x not in exclude_service)]
@@ -612,7 +596,7 @@ def validate_docker_service_mysql_usage(filename, tpl):
                 newfilename = \
                     os.path.normpath(os.path.join(os.path.dirname(incfile), f))
                 with open(newfilename, 'r') as newfile:
-                    newtmp = yaml.load(newfile.read(), Loader=yaml.SafeLoader)
+                    newtmp = yaml.load(newfile.read(), Loader=yaml.FullLoader)
                 read_all(newfile, newtmp)
 
     read_all(filename, tpl)
@@ -683,8 +667,7 @@ def validate_docker_service(filename, tpl):
                           % (section_name, filename))
                     return 1
 
-        if 'puppet_config' in role_data and \
-                VALIDATE_DOCKER_PUPPET_CONFIG_OVERRIDE.get(filename, True):
+        if 'puppet_config' in role_data:
             if validate_docker_service_mysql_usage(filename, tpl):
                 print('ERROR: could not validate use of mysql service for %s.'
                       % filename)
@@ -740,6 +723,16 @@ def validate_docker_service(filename, tpl):
                 print('ERROR: upgrade_tasks validation failed')
                 return 1
 
+        if 'fast_forward_upgrade_tasks' in role_data and role_data['fast_forward_upgrade_tasks']:
+            if validate_upgrade_tasks(role_data['fast_forward_upgrade_tasks']):
+                print('ERROR: fast_forward_upgrade_tasks validation failed')
+                return 1
+
+        if 'fast_forward_post_upgrade_tasks' in role_data and role_data['fast_forward_post_upgrade_tasks']:
+            if validate_upgrade_tasks(role_data['fast_forward_post_upgrade_tasks']):
+                print('ERROR: fast_forward_post_upgrade_tasks validation failed')
+                return 1
+
     if 'parameters' in tpl:
         for param in required_params:
             if param not in tpl['parameters']:
@@ -795,6 +788,16 @@ def validate_service(filename, tpl):
                 print('ERROR: upgrade_tasks validation failed')
                 return 1
 
+        if 'fast_forward_upgrade_tasks' in role_data and role_data['fast_forward_upgrade_tasks']:
+            if validate_upgrade_tasks(role_data['fast_forward_upgrade_tasks']):
+                print('ERROR: fast_forward_upgrade_tasks validation failed')
+                return 1
+
+        if 'fast_forward_post_upgrade_tasks' in role_data and role_data['fast_forward_post_upgrade_tasks']:
+            if validate_upgrade_tasks(role_data['fast_forward_post_upgrade_tasks']):
+                print('ERROR: fast_forward_post_upgrade_tasks validation failed')
+                return 1
+
     if 'parameters' in tpl:
         for param in required_params:
             if param not in tpl['parameters']:
@@ -941,10 +944,6 @@ def validate_service_hiera_interpol(f, tpl):
             if enter_lists and path[-1] != 0 and path[-2] != 'get_param':
                 continue
 
-            # Omit if it is not a hiera config setting
-            if path[1] in ['kolla_config']:
-                continue
-
             path_str = ';'.join(str(x) for x in path)
             # NOTE(bogdando): Omit foo_network keys looking like a network
             # name. The only exception is allow anything under
@@ -1052,7 +1051,7 @@ def validate(filename, param_map):
     retval = 0
     try:
         with open(filename, 'r') as f:
-            tpl = yaml.load(f.read(), Loader=yaml.SafeLoader)
+            tpl = yaml.load(f.read(), Loader=yaml.FullLoader)
 
         is_heat_template = 'heat_template_version' in tpl
 
@@ -1108,14 +1107,6 @@ def validate(filename, param_map):
                 filename.startswith('./roles/ComputeHCIOvsDpdk.yaml'):
             retval |= validate_hci_computehci_role(filename, tpl)
 
-        if filename.startswith('./roles/ControllerStorageNfs.yaml'):
-            exclude = [
-                'OS::TripleO::Services::CephNfs']
-            retval |= validate_controller_storage_nfs(filename, tpl, exclude)
-
-        if filename.startswith('./roles/ControllerStorageDashboard.yaml'):
-            retval |= validate_controller_dashboard(filename, tpl)
-
         if filename.startswith('./roles/ComputeOvsDpdk.yaml') or \
                 filename.startswith('./roles/ComputeSriov.yaml') or \
                 filename.startswith('./roles/ComputeOvsDpdkRT.yaml') or \
@@ -1244,11 +1235,11 @@ def validate_upgrade_tasks(upgrade_tasks):
 def validate_network_data_file(data_file_path):
     try:
         with open(data_file_path, 'r') as data_file:
-            data_file = yaml.load(data_file.read(), Loader=yaml.SafeLoader)
+            data_file = yaml.load(data_file.read(), Loader=yaml.FullLoader)
 
         base_file_path = os.path.dirname(data_file_path) + "/network_data.yaml"
         with open(base_file_path, 'r') as base_file:
-            base_file = yaml.load(base_file.read(), Loader=yaml.SafeLoader)
+            base_file = yaml.load(base_file.read(), Loader=yaml.FullLoader)
 
         retval = 0
         for n in base_file:
