@@ -88,6 +88,43 @@ function fqdn_check() {
   echo "SUCCESS"
 }
 
+# run chrony/ntpdate as available
+function _run_ntp_sync() {
+  local NTP_SERVER=$1
+  if ! type ntpdate 2>/dev/null; then
+    chronyd -Q "server $NTP_SERVER iburst"
+  else
+    ntpdate -qud $NTP_SERVER
+  fi
+}
+
+# Verify at least one time source is available.
+function ntp_check() {
+  NTP_SERVERS=$(hiera ntp::servers nil |tr -d '[],"')
+  if [[ "$NTP_SERVERS" != "nil" ]];then
+    echo -n "Testing NTP..."
+    NTP_SUCCESS=0
+    for NTP_SERVER in $NTP_SERVERS; do
+      set +e
+      NTPDATE_OUT=$(_run_ntp_sync $NTP_SERVER 2>&1)
+      NTPDATE_EXIT=$?
+      set -e
+      if [[ "$NTPDATE_EXIT" == "0" ]];then
+        NTP_SUCCESS=1
+        break
+      else
+        NTPDATE_OUT_FULL="$NTPDATE_OUT_FULL $NTPDATE_OUT"
+      fi
+    done
+    if  [[ "$NTP_SUCCESS" == "0" ]];then
+      echo "FAILURE"
+      echo "$NTPDATE_OUT_FULL"
+      exit 1
+    fi
+    echo "SUCCESS"
+  fi
+}
+
 if [[ "${validate_gateways_icmp,,}" == "true" ]];then
   ping_default_gateways
 fi
@@ -97,3 +134,6 @@ fi
 if [[ "${validate_fqdn,,}" == "true" ]];then
   fqdn_check
 fi
+if [[ "${validate_ntp,,}" == "true" ]];then
+  ntp_check
+fi
